Unit
    Uses
        cobra2d,
        keyset

Type sprites = Record
    id: Integer
    e: Element
EndType

Var
    gridImg, master: Element
    
    background: Element
    fpsCanvas: Element
    scoreCanvas: Element
    
    fm, mls, cfps, ticks: Integer ; export
    fd, frt: Real
    
    sprite: ^sprites
    spriteList: List of sprites
    
    spriteCounter: Integer = 0    

Function Hapi_1Rnd() : Real ; export
Begin  
    result = Rnd(0.0, 1.0)    
End
        
Procedure Hapi_DrawCell(x: Integer, y: Integer, cell: Integer) ; export
Begin
  
    Case cell of
        1:        ImageFrame(gridImg, 1)
        2:        ImageFrame(gridImg, 2)
        3:        ImageFrame(gridImg, 9)
        Default:    ImageFrame(gridImg, 0)
    EndCase    
    
    DrawImage(gridImg,x*16,y*16,background)
End

Function Hapi_CreateSprite() : Integer ; export
Begin
    
    sprite = NewItem(spriteList)
    If Not assigned(sprite) then MessageBox("Error assigning sprite!")
    sprite.id = spriteCounter
    sprite.e = CreateSharedSprite(master)
    //Hapi_SetSpriteFrame(sprite.id, 14)
    
    result = spriteCounter
    Inc(spriteCounter)
  
End

Procedure Hapi_SetSpriteFrame(id: Integer, frame: Integer) ; export
Begin
  
    sprite = GetSprite(id)
    SpriteFrame(sprite.e, frame)
    //ImageFrame(gridImg, frame)
    //DrawImage(gridImg,0,0,sprite.e)
    
End

Procedure Hapi_PositionSprite(id: Integer, x: Real, y: Real) ; export
Begin
  
    sprite = GetSprite(id)
    PositionSprite(sprite.e, x*16.0, y*16.0)
    
End

Procedure Hapi_FreeSprite(id: Integer) ; export
Begin

    sprite = GetSprite(id)
    FreeSprite(sprite.e)
    Free(sprite)
    
End

Procedure Hapi_DisplayStats(score: Integer, eaten: Integer) ; export
Begin
    CLS(ToRGBA(0,0,0,0), scoreCanvas)
    Text(0, 0,"Score: "+score+" Eaten: "+eaten, scoreCanvas)
End


Function Hapi_GetKeyDown(key: Integer) : Integer ; export
Begin
    If KeyDown(key) then result = 1 else result = 0
End

Function Hapi_GetDelta() : Real ; export
Begin
    result = fd
End

Function Hapi_GetTicks() : Integer ; export
Begin
    result = ticks
End

Function GetSprite(id: Integer) : ^sprites
Begin
  
    Loop sprite through spriteList
        If sprite.id = id then
            result = sprite
            exit
        Endif
    EndLoop
    
    MessageBox("No sprite with id "+id+"!")
    result = GetSprite(0)
    
End

Procedure pacmanInit() ; export
Var
    i: Integer
Begin

    background = CreateSprite(40*16, 30*16)
    
    fpsCanvas = CreateSprite(128, 32)
    PositionSprite(fpsCanvas, 500, 466) 
    PenColor(ToRGBA(255,255,255), fpsCanvas)
    SetFont("arial", 8, 0, fpsCanvas)
    
    scoreCanvas = CreateSprite(256, 32)
    PositionSprite(scoreCanvas, 20, 466)
    PenColor(ToRGBA(200,200,255), scoreCanvas)
    SetFont("arial", 8, 1, scoreCanvas) 
    
    gridImg = LoadMultiImage("grid.bmp", 16, 16)
    master = CreateMultiSprite(16, 16, 15)
    SpriteVisible(master, FALSE)
    
    For i = 0 to 14
        ImageFrame(gridImg, i)
        SpriteFrame(master, i)
        MaskImage(ToRGBA(128,0,128), 0, gridImg, i)
        DrawImage(gridImg, 0, 0, master)
    Next

End

Procedure SyncFramer() ; export
Var
    tmpFlFt: Real
Begin
    fm = Millisecs - mls
    mls = Millisecs
    fd = (fm / 1000.0) * 60.0
    fd = SmoothFrameDelta(fd)
    
    frt = frt + fd
    If (frt > 1.0)
        tmpFlFt = Floor(frt) 
        
        ticks = ticks + tmpFlFt
            
        frt = frt - ToReal(tmpFlFt)
    Endif 
End

Procedure ResetFramer() ; export
Var
    i: Integer
Begin
    frt = 0

    For i = 0 to 10
        SyncFramer()
    Next
End

Function CalcFps(m1: Integer, m2: Integer) : Integer ; export
Begin
    If ((m2 - m1) > 0) then
        result = (1000.0/(m2 - m1))
    Else
        result = 999
    Endif
End

Procedure renderFps(cfps: Integer) ; export
Begin
    CLS(ToRGBA(0,0,0,0), fpsCanvas)
    //Text(0, 0,"FPS: "+cfps+" delta: "+fd, fpsCanvas)
    Text(0, 0,"FPS: "+cfps+" ticks: "+ticks, fpsCanvas)
    //Text(0, 0,"FPS: "+cfps, fpsCanvas)
End

Function SmoothFrameDelta(fd: Real) : Real ; export
Var
    avg, fd1, fd2, fd3, fd4, fd5, fd6, fd7, fd8, fd9: Static Real
Begin

    avg = (fd1 + fd2 + fd3 + fd4 + fd5 + fd6 + fd7 + fd8 + fd9 + fd) / 10.0 
    
    fd1 = fd2
    fd2 = fd3
    fd3 = fd4
    fd4 = fd5
    fd5 = fd6
    fd6 = fd7
    fd7 = fd8
    fd8 = fd9
    fd9 = fd
    
    If fd > avg then fd = avg
    result = fd

End

Begin
End